/*'*****************************************************************************
' I2TZK  GCPU  -  18F4550 @ 20MHz LCD Graphic
'
' compile with MikroC Pro Pic at http://www.mikroe.com/mikroc/pic/
'
'*******************************************************************************
' v1.00 Skeleton for Graphic CPU h/w GCPU-0613  2013/10/20
'
'*******************************************************************************
' Graphic CPU h/w GCPU-0613 18F4550 pin configuration
'
' 2  RA0 - AN0  Analog input
' 3  RA1 - AN1  Analog input
' 4  RA2 - AN2  Not used
' 5  RA3 - AN3  Vref = LM385-2.5 Voltage Reference Diode
' 6  RA4 -      LCD CS2                       out
' 7  RA5 - AN4  Analog/Digital IO
' 8  RE0 - AN5  Analog/Digital IO
' 9  RE1 - AN6  LCD E                         out
' 10 RE2 - AN7  LCD RW                        out
'
' 33 RB0 -      bus I2C SDA                  in/out
' 34 RB1 - AN10 bus I2C SCL                  in/out
' 35 RB2 - AN8  pin 12 74HC595 / 2 74HC165   out
' 36 RB3 - AN9  pin 14 74HC595               out
' 37 RB4        BTN1 (bootloader)            in
' 38 RB5        BTN2                         in
' 39 RB6        ENC1                         in
' 40 RB7        ENC2                         in
'
' 15 RC0        Conuter Input                in
' 16 RC1        pin 11 75HC595               out
' 17 RC2 - CCP1 LCD CS1
'
' 23 RC4 - D- \
' 24 RC5 - D+ / USB
' 25 RC6 -      LCD RS
' 26 RC7 -      pin 1 74HC165 / BL on/off   out
'
' 18 Vusb 0.47uf to Ground
'
' 19 RD0 - SPP0 D0 \
' 20 RD1 - SPP1 D1 |
' 21 RD2        D2 |
' 22 RD3        D3 | LCD JDH128X64           out
' 27 RD4        D4 |
' 28 RD5        D5 |
' 29 RD6        D6 |
' 30 RD7        D7 /
'
' 1  RE3 -      Mclr h/w reset
' 11 Vdd
' 12 Vss
' 13 Osc1       20MHz Xtal oscillator
' 14 Osc2 - RA6 pin 9 of 74HC165                 in
' 31 Vss
' 32 Vdd
'******************************************************************************/

/*******************************************************************************
' All files to include
'******************************************************************************/
#include "built_in.h"

/*******************************************************************************
' Constant
'******************************************************************************/

  // TMR0 initialized at 10 mSec
  const unsigned TMR0_PRELOAD    = 0x8AD1 ;                    //  TMR0 preload [10mSec prescaler 1:4] +/- tmrAdj2 [0-100]
  const unsigned char TMR0_PRELOAD_LO = 0xD1;
  const unsigned char TMR0_PRELOAD_HI = 0x8A;
  const unsigned char TMR0_PRESCALE    = 0b00000001;           // T0CON  Stop timer, 16bit, prescale assigned, prescaler 1:4
  const unsigned char TMR0_MULT       = 100;                   // --> TMR0_PRELOAD x TMR0_MULT = 1 Sec

  // ------------------------------------------------------
  // GLCD Picture name: FoxBanner.bmp
  // GLCD Model: KS0108 128x64
  // ------------------------------------------------------
  const code char FoxBanner[1024] = {
  255,   1,   1,  65,  65, 129, 129,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1, 129, 193, 129,   1,   1,   1,   1,   1,   1,   1, 129, 129, 129, 129, 129, 129, 129,   1,   1,   1, 129, 129, 129,   1,   1,   1, 129, 129, 129,   1, 129, 129, 129,   1,   1,   1,   1,   1,   1,   1,   1,   1, 129, 129, 129, 129, 129,   1,   1,   1, 129, 129, 129, 129, 129, 129, 129,   1, 129, 129, 129, 129, 129,   1,   1,   1, 129, 129, 129, 129, 129, 129, 129,   1,   1,   1, 129, 129,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1, 255,
  255,   0,  15, 255, 192,  60, 253,  27,  18,  52, 236, 216,  48,  32, 192, 128,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0, 128, 128,  64,  64,  32,  32,  16,  24,   8,   6,   3, 120,   0, 239,   0,   0,   0,   0,   0,   0,   0, 128, 255, 136, 136,  28,   0,   1,   0,  62,  65, 128, 128, 128,  65,  62,   0, 128, 193, 182,   8, 182, 193, 128,   0,   0,   0,   0,   0,   0,   0,   0,   0, 128, 255, 128, 128, 128,  65,  62,   0, 128, 255, 136, 136, 156, 128, 193,   0, 128, 128, 255, 128, 128, 128, 192,   0,   1,   0, 128, 255, 128,   0,   1,   0, 128, 240, 172,  35, 172, 240, 128,   0,   0,   0,   0,   0,   0,   0, 255,
  255,   0,   0,   0,   7,  62, 127, 207, 252, 240,   0,   1,   3,  15,   0,   1,   0,   2,   3,   3,   0,  48,  97, 129,   1,   1,   1,  51,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,  96,  48,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0, 255,
  255,   0,   0,   0,   0,   0,   0, 249, 255,   2,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   6,   0,   0,   8,   0,   0,   0,   0,   0,   0, 128,  24,  64, 192,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0, 255,
  255,   0,   0,   0,   0,   0,   0, 255, 255, 255, 135, 194, 128,   0, 192, 240, 248, 128, 128,   0,   1,   3,   6,  14,  28,  56,  48,  96,   0,   0,   0,  48,  30,  15, 128,   0,  64,  31,   4,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0, 255,
  255,   0,   0,   0,   0,   0,   0,   0,   3,  15,  31,  63, 108, 199, 153,   9, 159,  15,  75,  75,  35,   1,   0,   3,  34,  34,  96,  64, 192, 128, 128,   0,   0,   1,   2,   0, 240, 240,  48,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   2, 254, 254,   2,   2,   0,   0,   0,   4, 134, 194,  98,  62,  28,   0,   0,   6,   6,   2, 254, 254,   2,   6,   6,   0,   6,   6, 194, 226,  58,  30,   6,   2,   0,   2, 254, 254,  98, 248, 158,   6,   2,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0, 255,
  255,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   3,   6,  12,  12,  16,  48,  16,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   2,   3,   3,   2,   2,   0,   0,   0,   3,   3,   2,   2,   2,   2,   0,   0,   0,   0,   2,   3,   3,   2,   0,   0,   0,   2,   3,   3,   2,   2,   2,   3,   3,   0,   2,   3,   3,   2,   0,   3,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0, 255,
  255, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 255
  };
/*******************************************************************************
' PROGRAM variables Regular variables
'******************************************************************************/
// Glcd module connections -  CS1 & CS2 swapped
  char GLCD_DataPort at PORTD;
  sbit GLCD_CS1 at LATA4_bit;
  sbit GLCD_CS2 at LATC2_bit;
  sbit GLCD_RS  at LATC6_bit;
  sbit GLCD_RW  at LATE2_bit;
  sbit GLCD_EN  at LATE1_bit;
  sbit GLCD_RST at LATB5_bit;
  sbit GLCD_BL at PORTC.B7;                          // backligh 0:ON, 1:OFF
  
  sbit GLCD_CS1_Direction at TRISA4_bit;
  sbit GLCD_CS2_Direction at TRISC2_bit;
  sbit GLCD_RS_Direction  at TRISC6_bit;
  sbit GLCD_RW_Direction  at TRISE2_bit;
  sbit GLCD_EN_Direction  at TRISE1_bit;
  sbit GLCD_RST_Direction at TRISB5_bit;

  // 74HC595 Serial IN / Parallel OUT
  sbit CK_595   at LATB2_bit;                         // out pin 12 74HC595
  sbit DATA_595 at LATB3_bit;                         // out pin 14 74HC595
  sbit LOAD_595 at PORTC.B1;                         // out pin 11 74HC595
  // 74HC165 Parallel IN / Serial OUT
  sbit CK_165   at LATB2_bit;                         // out pin 2 74HC165
  sbit DATA_165 at PORTA.B6;                         // in  pin 9 74HC165
  sbit LOAD_165 at PORTC.B7;                         // out pin 1 74HC165


// frequency counter
  sbit COUNTER_Direction at TRISC.B0;
  sbit COUNTER at LATC0_bit;
  
// global isr debounching inputs
  char switchState;
  char switchFlag;
  char inPort;
  
// global isr detect encoder direction
  char encDirection;                         // 0:stop 1:clockwise  -1:counterclockwise
  static const unsigned short enc_states [] ={0,-1,1,0,1,0,0,-1,-1,0,0,1,0,1,-1,0}; //encoder lookup table

// global variable generic usage
  char boardConfig;
  char lcdBuffer[20];
  int counter1;
  int counter2;
  
/*******************************************************************************
' Declare needed functions
'******************************************************************************/
void main();
/*'-----------------------------------------------------------------------------
  ISR routine called every 10mS by TMR0 interrupt
' uses a vertical counter to debounce all inputs
' returns a flag with changes & debounced state of all inputs
' used to bebounce BTN1 & BTN2
'----------------------------------------------------------------------------*/
unsigned char debounce(unsigned char sample, unsigned char *toggle){
  static unsigned char state, cnt0, cnt1;
  unsigned char delta;

  delta = sample ^ state;
  cnt1 = (cnt1 ^ cnt0) & delta;
  cnt0 = ~cnt0 & delta;
  *toggle = delta & ~(cnt0 | cnt1);
  state ^= *toggle;

  return state;
}
/*'-----------------------------------------------------------------------------
' debounce and detect the encoder direction
'----------------------------------------------------------------------------*/
void encoderDetect(){
  static unsigned short encValOld = 3;               //lookup table index and initial state
  static signed char encVal;

  encValOld <<=2;                                      //remember previous state
  inPort >>=6;
  encValOld |= inPort;                               //create index
  encVal += enc_states[encValOld & 0x0F];            //get direction

  if( encval < -3 ) {                                   //1 step clockwise
    encval = 0;
    encDirection = 0x01;
    }
  else if( encVal > 3 ) {                             //1 step counterclockwise
    encVal = 0;
    encDirection = 0xFF;
    }
  else {
    asm nop                                            // encVal = 0
    encDirection = 0x00;
    }
  }

/*'---------------------------------------------------------
' return the configuration byte read from the parallel in/
' serial out register 74HC165
'---------------------------------------------------------*/
unsigned short read_HC165() {
  char dataRead;            // working value of switches
  char i;

  LOAD_165 = 1;                       // enable shift register
  dataRead = 0;                       // clear data in
  for (i = 0; i < 8; i++)
    {
    dataRead <<= 1;                   // make room for the next bit
    CK_165 = 0;                       // drop the CLK signal
    dataRead |= DATA_165;             // get the bit
    asm nop;                          // small delay
    CK_165 = 1;                       // make the next bit available
    }
  CK_165 = 0;                         // leave clock low
  LOAD_165 = 0;                       // disable shift register
  //GLCD_BL = 0;                       // GLCD backlight share 165 CS

  // pcb error: dip switch 3(165/D2) & 4(165/D3) are swapped
  i = dataRead;
  dataRead.B2 = i.B3;
  dataRead.B3 = i.B2;

  return dataRead;                   // dataRead has the 8 inputs.
}


/*'---------------------------------------------------------
' write a byte to the serial in / parelle out register 74HC595
'---------------------------------------------------------*/
void write_HC595(char one_digit) {
  char x;

  for (x=0; x<=7; ++x) {
    if (one_digit & 0x80) DATA_595 = 1;   // out 1 bit
    else DATA_595 = 0;
    one_digit <<= 1;                      // prepare next bit
    LOAD_595 = 1;                         // load internal register
    asm nop;                              // small delay
    LOAD_595 = 0;
  }
  CK_595 = 1;                             // clock pulse to output register
  asm nop;
  CK_595 = 0;
  LOAD_595 = 0;                           // disable the register
}
/*'---------------------------------------------------------
' HIGH priority interrupt
'---------------------------------------------------------*/
void interrupt() 
{
  if (INTCON.TMR0IF) {
    /*'---------------------------------------------------------
    ' 10mSec TMR0 expired
    '---------------------------------------------------------*/
    inPort = PORTB & 0b11110000;                            // read buttons & encoder
    switchState = debounce(inPort, &switchFlag);            // debounce all inputs
    encoderDetect();                                        // debounce & detect direction
    TMR0H = TMR0_PRELOAD_HI;                                // poich TMR0  un registro bufferato
    TMR0L = TMR0_PRELOAD_LO;                                // scrivere prima H poi Lr
    INTCON.TMR0IF = 0;                                      // reset interrupt flag
    }
}
  
/*'---------------------------------------------------------
' LOW priority interrupt
'---------------------------------------------------------*/
void interrupt_low() {
  // put here your code
  }

/*'---------------------------------------------------------
' Remap interrupt vector to use with Microchip boot loader
'---------------------------------------------------------*/
void Vectors() org 0x1000 {
     asm {
       goto   _main                   //0x1000
       nop
       nop
       goto   _interrupt              //0x1008
       nop
       nop
       nop
       nop
       nop
       nop
       goto   _interrupt_low          //0x1018
     }
  }

/*'*******************************************************************************
' DATA CODE tables
'********************************************************************************/
    const code char AUTHOR_ID[]  = "i2tzk";
    const code char PROJECT_ID[] = "GCPU-0613  ";
    const code char FW_ID[]      =  "01.00";
/*'******************************************************************************/

void main() {
  //--------------------------------------------------------------------------
  //Make sure entire program starts at 0x1000 which is where the boot loader finishes
  OrgAll(0x1000);
  //ASM Command to avoid optimization which ensures Vectors will be remapped correctly
  asm { goto SkipVectors }
  Vectors();
  asm { SkipVectors: }
/*'-----------------------------------------------------------------------------
  '  TRIS => 1 input, 0 output
  '----------------------------------------------------------------------------*/
  TRISA = 0b11101111;                               // RA 0,1,2,3,5,6,7 in, RA 4 out
  PORTA = 0;

  TRISB = 0b11110000;                               // RB 4,5,6,7 in, RB 0,1,2,3 out
  PORTB = 0;                                        //

  TRISC = 0b00000001;                               // RC 0 in, RC 1,2,3,4,5,6,7 out
  PORTC = 0;

  TRISD = 0b00000000;                               // RD 0,1,2,3,4,5,6,7 out
  PORTD = 0;
  /*'-------------------------------------------------------------
  ' Inizialize ADC
  '-------------------------------------------------------------*/
  ADCON0  = 0b00000001;                                   // AD enabled
  ADCON1  = 0b00011001;                                   // AN0-AN5 analog input, AN6-AN12 digital I/O
  ADCON2 =  0b10011110;                                   // rigth justified, Fosc/64, Tad6                                                    // AN3 Vref+
  /*'-------------------------------------------------------------
  ' Inizialize Timer & Interrupts
  '-------------------------------------------------------------
  ' TMR0  -> time base gate
  '-------------------------------------------------------------*/
  T0CON = TMR0_PRESCALE;                                  // stop timer
  TMR0H = TMR0_PRELOAD_HI;                                // poich TMR0  un registro bufferato
  TMR0L = TMR0_PRELOAD_LO;                                // scrivere prima H poi L
  INTCON2.TMR0IP = 1;                                     // Set TMR0 As a HIGH priority interrupt source
  INTCON.TMR0IE = 1;                                      // enable overflow interrupt
  INTCON.TMR0IF = 0;

  /*'-------------------------------------------------------------
  ' let interrupt on change disabled
  ' read and debounce BTN1, BTN2 & encoder polling every 10mS
  '-------------------------------------------------------------*/
  inPort = PORTB;                                        // reset switch & encoder port
/*INTCON2.RBIP = 1;                                     // RB Port Change Interrupt HIGH priority
  INTCON.RBIF = 0;                                      // clear interrupt flag on change port B
  INTCON.RBIE = 1;                                      // enable PortB interrupt on change*/
  /*'------------------------------------------------------------
  ' read board configuration DIPs
  '------------------------------------------------------------*/
  boardConfig = read_HC165();
  /*'------------------------------------------------------------
  ' inizialize GLCD & msg di HELLO
  '------------------------------------------------------------*/
  GLCD_BL = 0;                                          // glcd BACKLIGHT on
  Glcd_Init();                                          // Initialize GLCD
  Glcd_Image(FoxBanner);                                // FoxDelta Logo
  Delay_ms(2000);

  // Choose font, see __Lib_GLCDFonts.c in Uses folder
  /* Glcd_Set_Font(Character8x7, 8, 7, 32);                  // Change font
  Glcd_Set_Font(System3x5, 3, 5, 32);                    // Change font
  Glcd_Set_Font(font5x7, 5, 7, 32);                      // Change font
  Glcd_Set_Font(FontSystem5x7_v2, 5, 7, 32);             // Change font*/

  Glcd_Fill(0x00);                                        // Clear GLCD
  Glcd_Set_Font(Font_Glcd_Character8x7, 8, 7, 32);        // select font
  /*'------------------------------------------------------------
  ' enable interrupt
  '------------------------------------------------------------*/
  RCON.IPEN = 1;                                          // Enable priority interrupts.
  INTCON.GIEH = 1;                                        // Set the global interrupt enable bits
  INTCON.GIEL = 1;
  T0CON.TMR0ON = 1;                                       // start TMR0
  Glcd_Write_Text("Push BTN", 0, 0, 1);
  Glcd_Write_Text("Rotate ENC", 0, 1, 1);
  counter1 = 0;
  counter2 = 0;
  while (1) {                                             // Endless loop
      
    // BTN1 & BTN2
    if (switchFlag != 0) {
     if (switchFlag.B4 == 1) {
          if (switchState.B4 == 0) {
            Glcd_Write_Text("BTN 1 down", 0, 3, 1);
            // here your code for button down action
            }
          else {
            Glcd_Write_Text("BTN 1 up     ", 0, 3, 1);
            // here your code for button up action
            }
        }
     else if (switchFlag.B5 == 1) {
          if (switchState.B5 == 0) {
             Glcd_Write_Text("BTN 2 down", 0, 3, 1);
             // here your code for button down action
            }
          else {
            Glcd_Write_Text("BTN 2 up     ", 0, 3, 1);
            // here your code for button up action
            }
        }
     switchFlag = 0;
    }
    // Encoder
    if (encDirection == 0x01) {
      encDirection = 0;
      // here your code for encoder clockwise action
      Glcd_Write_Text(">         ", 0, 3, 1 );
      counter1++ ;
      counter2 = 0;
      sprintf(lcdBuffer, "%d", counter1);               // format text
      Glcd_Write_Text(lcdBuffer, 10, 3, 1);             // write new text
      }
    else if (encDirection == 0xFF) {
      encDirection = 0;
      // here your code for encoder counterclockwise action
      Glcd_Write_Text("<         ", 0, 3, 1 );
      counter2++ ;
      counter1 = 0;
      sprintf(lcdBuffer, "%d", counter2);               // format text
      Glcd_Write_Text(lcdBuffer, 10, 3, 1);             // write new text
      }
    else {
        // here your code for NO encoder action
        // Glcd_Write_Text("          ", 0, 3, 1 );
      }
      
    // from here your main code
    boardConfig = read_HC165();
    write_HC595(boardConfig);                             // testing parallel out port
    
    }
  }